@@ -9,7 +9,7 @@
 #include "mpl_kind_code.h"
 #include <algorithm>
 
-
+namespace contourpy {
 namespace mpl2014 {
 
 // Point indices from current quad index.
@@ -83,32 +83,15 @@ namespace mpl2014 {
 
 
 
-QuadEdge::QuadEdge()
-    : quad(-1), edge(Edge_None)
-{}
-
 QuadEdge::QuadEdge(index_t quad_, Edge edge_)
     : quad(quad_), edge(edge_)
 {}
 
-bool QuadEdge::operator<(const QuadEdge& other) const
-{
-    if (quad != other.quad)
-        return quad < other.quad;
-    else
-        return edge < other.edge;
-}
-
 bool QuadEdge::operator==(const QuadEdge& other) const
 {
     return quad == other.quad && edge == other.edge;
 }
 
-bool QuadEdge::operator!=(const QuadEdge& other) const
-{
-    return !operator==(other);
-}
-
 std::ostream& operator<<(std::ostream& os, const QuadEdge& quad_edge)
 {
     return os << quad_edge.quad << ' ' << quad_edge.edge;
@@ -116,9 +99,6 @@ std::ostream& operator<<(std::ostream& os, const QuadEdge& quad_edge)
 
 
 
-XY::XY()
-{}
-
 XY::XY(const double& x_, const double& y_)
     : x(x_), y(y_)
 {}
@@ -128,40 +108,6 @@ bool XY::operator==(const XY& other) const
     return x == other.x && y == other.y;
 }
 
-bool XY::operator!=(const XY& other) const
-{
-    return x != other.x || y != other.y;
-}
-
-XY XY::operator*(const double& multiplier) const
-{
-    return XY(x*multiplier, y*multiplier);
-}
-
-const XY& XY::operator+=(const XY& other)
-{
-    x += other.x;
-    y += other.y;
-    return *this;
-}
-
-const XY& XY::operator-=(const XY& other)
-{
-    x -= other.x;
-    y -= other.y;
-    return *this;
-}
-
-XY XY::operator+(const XY& other) const
-{
-    return XY(x + other.x, y + other.y);
-}
-
-XY XY::operator-(const XY& other) const
-{
-    return XY(x - other.x, y - other.y);
-}
-
 std::ostream& operator<<(std::ostream& os, const XY& xy)
 {
     return os << '(' << xy.x << ' ' << xy.y << ')';
@@ -335,8 +281,8 @@ Mpl2014ContourGenerator::Mpl2014ContourGenerator(
       _ny(_z.ndim() > 0 ? _z.shape(0) : 0),
       _n(_nx*_ny),
       _corner_mask(corner_mask),
-      _x_chunk_size(x_chunk_size > 0 ? std::min(x_chunk_size, _nx-1) : _nx-1),
-      _y_chunk_size(y_chunk_size > 0 ? std::min(y_chunk_size, _ny-1) : _ny-1),
+      _x_chunk_size(calc_chunk_size(_nx, x_chunk_size)),
+      _y_chunk_size(calc_chunk_size(_ny, y_chunk_size)),
       _nxchunk(calc_chunk_count(_nx, _x_chunk_size)),
       _nychunk(calc_chunk_count(_ny, _y_chunk_size)),
       _chunk_count(_nxchunk*_nychunk),
@@ -365,7 +311,7 @@ Mpl2014ContourGenerator::Mpl2014ContourGenerator(
     }
 
     if (x_chunk_size < 0 || y_chunk_size < 0)
-        throw std::invalid_argument("chunk_size cannot be negative");
+        throw std::invalid_argument("x_chunk_size and y_chunk_size cannot be negative");
 
     init_cache_grid(mask);
 }
@@ -500,13 +446,11 @@ void Mpl2014ContourGenerator::append_contour_to_vertices_and_codes(
     contour.delete_contour_lines();
 }
 
-index_t Mpl2014ContourGenerator::calc_chunk_count(
-    index_t point_count, index_t chunk_size) const
+index_t Mpl2014ContourGenerator::calc_chunk_count(index_t point_count, index_t chunk_size)
 {
-    assert(point_count > 0 && "point count must be positive");
-    assert(chunk_size > 0 && "Chunk size must be positive");
+    // Accepts any values for point_count and chunk_size.
 
-    if (chunk_size > 0) {
+    if (chunk_size > 0 && point_count > 1) {
         index_t count = (point_count-1) / chunk_size;
         if (count*chunk_size < point_count-1)
             ++count;
@@ -518,6 +462,16 @@ index_t Mpl2014ContourGenerator::calc_chunk_count(
         return 1;
 }
 
+index_t Mpl2014ContourGenerator::calc_chunk_size(index_t point_count, index_t chunk_size)
+{
+    // Accepts any values for point_count and chunk_size.
+    index_t ret = point_count - 1;
+    if (chunk_size > 0)
+        ret = std::min(chunk_size, ret);
+
+    return std::max<index_t>(ret, 1);
+}
+
 void Mpl2014ContourGenerator::edge_interp(
     const QuadEdge& quad_edge, const double& level, ContourLine& contour_line)
 {
@@ -1725,7 +1679,7 @@ bool Mpl2014ContourGenerator::start_line(
     return VISITED(quad,1);
 }
 
-void Mpl2014ContourGenerator::write_cache(bool grid_only) const
+/*void Mpl2014ContourGenerator::write_cache(bool grid_only) const
 {
     std::cout << "-----------------------------------------------" << std::endl;
     for (index_t quad = 0; quad < _n; ++quad)
@@ -1753,6 +1707,7 @@ void Mpl2014ContourGenerator::write_cache_quad(
             << VISITED_CORNER(quad);
     }
     std::cout << std::endl;
-}
+}*/
 
 } // namespace mpl2014
+} // namespace contourpy
