@@ -1,5 +1,8 @@
+from __future__ import annotations
+
 from functools import reduce
 from operator import add
+from typing import TYPE_CHECKING, cast
 
 import numpy as np
 from numpy.testing import assert_allclose, assert_array_equal
@@ -10,9 +13,12 @@ from contourpy.util.data import random, simple
 
 from . import util_test
 
+if TYPE_CHECKING:
+    import contourpy._contourpy as cpy
+
 
 @pytest.fixture
-def two_outers_one_hole():
+def two_outers_one_hole() -> tuple[cpy.CoordinateArray, ...]:
     x, y = np.meshgrid([0., 1., 2., 3.], [0., 1., 2.])
     z = np.array([[1.5, 1.5, 0.9, 0.0],
                   [1.5, 2.8, 0.4, 0.8],
@@ -21,15 +27,22 @@ def two_outers_one_hole():
 
 
 @pytest.fixture
-def xyz_chunk_test():
+def xyz_chunk_test() -> tuple[cpy.CoordinateArray, ...]:
     x, y = np.meshgrid(np.arange(5), np.arange(5))
     z = 0.5*np.abs(y - 2) + 0.1*(x - 2)
     return x, y, z
 
 
+@pytest.mark.parametrize("name", util_test.all_names())
+def test_filled_decreasing_levels(name: str) -> None:
+    cont_gen = contour_generator(z=[[0, 1], [2, 3]], name=name, fill_type=FillType.OuterCode)
+    with pytest.raises(ValueError, match="upper and lower levels are the wrong way round"):
+        cont_gen.filled(2.0, 1.0)
+
+
 @pytest.mark.image
 @pytest.mark.parametrize("name, fill_type", util_test.all_names_and_fill_types())
-def test_filled_simple(name, fill_type):
+def test_filled_simple(name: str, fill_type: FillType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -52,7 +65,7 @@ def test_filled_simple(name, fill_type):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, fill_type", util_test.all_names_and_fill_types())
-def test_filled_simple_chunk(name, fill_type):
+def test_filled_simple_chunk(name: str, fill_type: FillType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -81,7 +94,7 @@ def test_filled_simple_chunk(name, fill_type):
 @pytest.mark.threads
 @pytest.mark.parametrize("fill_type", FillType.__members__.values())
 @pytest.mark.parametrize("thread_count", util_test.thread_counts())
-def test_filled_simple_chunk_threads(fill_type, thread_count):
+def test_filled_simple_chunk_threads(fill_type: FillType, thread_count: int) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -108,7 +121,7 @@ def test_filled_simple_chunk_threads(fill_type, thread_count):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, fill_type", util_test.all_names_and_fill_types())
-def test_filled_simple_no_corner_mask(name, fill_type):
+def test_filled_simple_no_corner_mask(name: str, fill_type: FillType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -131,7 +144,7 @@ def test_filled_simple_no_corner_mask(name, fill_type):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, fill_type", util_test.all_names_and_fill_types())
-def test_filled_simple_no_corner_mask_chunk(name, fill_type):
+def test_filled_simple_no_corner_mask_chunk(name: str, fill_type: FillType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -161,7 +174,7 @@ def test_filled_simple_no_corner_mask_chunk(name, fill_type):
 @pytest.mark.threads
 @pytest.mark.parametrize("fill_type", FillType.__members__.values())
 @pytest.mark.parametrize("thread_count", util_test.thread_counts())
-def test_filled_simple_no_corner_mask_chunk_threads(fill_type, thread_count):
+def test_filled_simple_no_corner_mask_chunk_threads(fill_type: FillType, thread_count: int) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -190,7 +203,7 @@ def test_filled_simple_no_corner_mask_chunk_threads(fill_type, thread_count):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name", util_test.corner_mask_names())
-def test_filled_simple_corner_mask(name):
+def test_filled_simple_corner_mask(name: str) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -214,7 +227,7 @@ def test_filled_simple_corner_mask(name):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name", util_test.corner_mask_names())
-def test_filled_simple_corner_mask_chunk(name):
+def test_filled_simple_corner_mask_chunk(name: str) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -245,7 +258,7 @@ def test_filled_simple_corner_mask_chunk(name):
 @pytest.mark.threads
 @pytest.mark.parametrize("fill_type", FillType.__members__.values())
 @pytest.mark.parametrize("thread_count", util_test.thread_counts())
-def test_filled_simple_corner_mask_chunk_threads(fill_type, thread_count):
+def test_filled_simple_corner_mask_chunk_threads(fill_type: FillType, thread_count: int) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -275,7 +288,7 @@ def test_filled_simple_corner_mask_chunk_threads(fill_type, thread_count):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name", util_test.quad_as_tri_names())
-def test_filled_simple_quad_as_tri(name):
+def test_filled_simple_quad_as_tri(name: str) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -299,7 +312,7 @@ def test_filled_simple_quad_as_tri(name):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, fill_type", util_test.all_names_and_fill_types())
-def test_filled_random(name, fill_type):
+def test_filled_random(name: str, fill_type: FillType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -322,7 +335,7 @@ def test_filled_random(name, fill_type):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, fill_type", util_test.all_names_and_fill_types())
-def test_filled_random_chunk(name, fill_type):
+def test_filled_random_chunk(name: str, fill_type: FillType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -365,7 +378,7 @@ def test_filled_random_chunk(name, fill_type):
 @pytest.mark.threads
 @pytest.mark.parametrize("fill_type", FillType.__members__.values())
 @pytest.mark.parametrize("thread_count", util_test.thread_counts())
-def test_filled_random_chunk_threads(fill_type, thread_count):
+def test_filled_random_chunk_threads(fill_type: FillType, thread_count: int) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -400,7 +413,7 @@ def test_filled_random_chunk_threads(fill_type, thread_count):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, fill_type", util_test.all_names_and_fill_types())
-def test_filled_random_no_corner_mask(name, fill_type):
+def test_filled_random_no_corner_mask(name: str, fill_type: FillType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -423,7 +436,7 @@ def test_filled_random_no_corner_mask(name, fill_type):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name, fill_type", util_test.all_names_and_fill_types())
-def test_filled_random_no_corner_mask_chunk(name, fill_type):
+def test_filled_random_no_corner_mask_chunk(name: str, fill_type: FillType) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -466,7 +479,7 @@ def test_filled_random_no_corner_mask_chunk(name, fill_type):
 @pytest.mark.threads
 @pytest.mark.parametrize("fill_type", FillType.__members__.values())
 @pytest.mark.parametrize("thread_count", util_test.thread_counts())
-def test_filled_random_no_corner_mask_chunk_threads(fill_type, thread_count):
+def test_filled_random_no_corner_mask_chunk_threads(fill_type: FillType, thread_count: int) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -502,7 +515,7 @@ def test_filled_random_no_corner_mask_chunk_threads(fill_type, thread_count):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name", util_test.corner_mask_names())
-def test_filled_random_corner_mask(name):
+def test_filled_random_corner_mask(name: str) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -526,7 +539,7 @@ def test_filled_random_corner_mask(name):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name", util_test.corner_mask_names())
-def test_filled_random_corner_mask_chunk(name):
+def test_filled_random_corner_mask_chunk(name: str) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -563,7 +576,7 @@ def test_filled_random_corner_mask_chunk(name):
 @pytest.mark.threads
 @pytest.mark.parametrize("fill_type", FillType.__members__.values())
 @pytest.mark.parametrize("thread_count", util_test.thread_counts())
-def test_filled_random_corner_mask_chunk_threads(fill_type, thread_count):
+def test_filled_random_corner_mask_chunk_threads(fill_type: FillType, thread_count: int) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -593,7 +606,7 @@ def test_filled_random_corner_mask_chunk_threads(fill_type, thread_count):
 
 @pytest.mark.image
 @pytest.mark.parametrize("name", util_test.quad_as_tri_names())
-def test_filled_random_quad_as_tri(name):
+def test_filled_random_quad_as_tri(name: str) -> None:
     from contourpy.util.mpl_renderer import MplTestRenderer
 
     from .image_comparison import compare_images
@@ -616,7 +629,11 @@ def test_filled_random_quad_as_tri(name):
 
 @pytest.mark.parametrize("fill_type", FillType.__members__.values())
 @pytest.mark.parametrize("name", ["serial", "threaded"])
-def test_return_by_fill_type(two_outers_one_hole, name, fill_type):
+def test_return_by_fill_type(
+    two_outers_one_hole: tuple[cpy.CoordinateArray, ...],
+    name: str,
+    fill_type: FillType,
+) -> None:
     x, y, z = two_outers_one_hole
     cont_gen = contour_generator(x, y, z, name=name, fill_type=fill_type)
 
@@ -627,50 +644,96 @@ def test_return_by_fill_type(two_outers_one_hole, name, fill_type):
 
     util_test.assert_filled(filled, fill_type)
 
-    if fill_type in (FillType.OuterCode, FillType.OuterOffset):
-        points = filled[0]
-        assert len(points) == 2
+    # Helper functions to avoid code duplication for the different FillTypes.
+    def assert_outer_points(points: list[cpy.PointArray]) -> None:
+        assert isinstance(points, list) and len(points) == 2
         assert points[0].shape == (13, 2)
         assert points[1].shape == (4, 2)
         assert_array_equal(points[0][0], points[0][7])
         assert_array_equal(points[0][8], points[0][12])
         assert_array_equal(points[1][0], points[1][3])
-        if fill_type == FillType.OuterCode:
-            codes = filled[1]
-            assert_array_equal(codes[0], [1, 2, 2, 2, 2, 2, 2, 79, 1, 2, 2, 2, 79])
-            assert_array_equal(codes[1], [1, 2, 2, 79])
-        else:
-            offsets = filled[1]
-            assert_array_equal(offsets[0], [0, 8, 13])
-            assert_array_equal(offsets[1], [0, 4])
-    else:
-        points = filled[0]
-        assert len(points) == 1
-        points = points[0]
+
+    def assert_chunk_points(points_or_none: list[cpy.PointArray | None]) -> None:
+        assert isinstance(points_or_none, list) and len(points_or_none) == 1
+        assert points_or_none[0] is not None
+        points = points_or_none[0]
         assert points.shape == (17, 2)
         assert_array_equal(points[0], points[7])
         assert_array_equal(points[8], points[12])
         assert_array_equal(points[13], points[16])
-        if fill_type in (FillType.ChunkCombinedCode, FillType.ChunkCombinedCodeOffset):
-            codes = filled[1][0]
-            assert_array_equal(codes, [1, 2, 2, 2, 2, 2, 2, 79, 1, 2, 2, 2, 79, 1, 2, 2, 79])
-        else:
-            offsets = filled[1][0]
-            assert_array_equal(offsets, [0, 8, 13, 17])
 
-        if fill_type == FillType.ChunkCombinedCodeOffset:
-            outer_offsets = filled[2][0]
-            assert_array_equal(outer_offsets, [0, 13, 17])
-        elif fill_type == FillType.ChunkCombinedOffsetOffset:
-            outer_offsets = filled[2][0]
-            assert_array_equal(outer_offsets, [0, 2, 3])
+    def assert_chunk_codes(codes_or_none: list[cpy.CodeArray | None]) -> None:
+        assert isinstance(codes_or_none, list) and len(codes_or_none) == 1
+        assert codes_or_none[0] is not None
+        codes = codes_or_none[0]
+        assert_array_equal(codes, [1, 2, 2, 2, 2, 2, 2, 79, 1, 2, 2, 2, 79, 1, 2, 2, 79])
+
+    def assert_chunk_offsets(offsets_or_none: list[cpy.OffsetArray | None]) -> None:
+        assert isinstance(offsets_or_none, list) and len(offsets_or_none) == 1
+        assert offsets_or_none[0] is not None
+        offsets = offsets_or_none[0]
+        assert_array_equal(offsets, [0, 8, 13, 17])
+
+    if fill_type == FillType.OuterCode:
+        if TYPE_CHECKING:
+            filled = cast(cpy.FillReturn_OuterCode, filled)
+        assert_outer_points(filled[0])
+        codes = filled[1]
+        assert isinstance(codes, list) and len(codes) == 2
+        assert_array_equal(codes[0], [1, 2, 2, 2, 2, 2, 2, 79, 1, 2, 2, 2, 79])
+        assert_array_equal(codes[1], [1, 2, 2, 79])
+    elif fill_type == FillType.OuterOffset:
+        if TYPE_CHECKING:
+            filled = cast(cpy.FillReturn_OuterOffset, filled)
+        assert_outer_points(filled[0])
+        offsets = filled[1]
+        assert isinstance(offsets, list) and len(offsets) == 2
+        assert_array_equal(offsets[0], [0, 8, 13])
+        assert_array_equal(offsets[1], [0, 4])
+    elif fill_type == FillType.ChunkCombinedCode:
+        if TYPE_CHECKING:
+            filled = cast(cpy.FillReturn_ChunkCombinedCode, filled)
+        assert_chunk_points(filled[0])
+        assert_chunk_codes(filled[1])
+    elif fill_type == FillType.ChunkCombinedOffset:
+        if TYPE_CHECKING:
+            filled = cast(cpy.FillReturn_ChunkCombinedOffset, filled)
+        assert_chunk_points(filled[0])
+        assert_chunk_offsets(filled[1])
+    elif fill_type == FillType.ChunkCombinedCodeOffset:
+        if TYPE_CHECKING:
+            filled = cast(cpy.FillReturn_ChunkCombinedCodeOffset, filled)
+        assert_chunk_points(filled[0])
+        assert_chunk_codes(filled[1])
+
+        outer_offsets_or_none = filled[2]
+        assert isinstance(outer_offsets_or_none, list) and len(outer_offsets_or_none) == 1
+        assert outer_offsets_or_none[0] is not None
+        assert_array_equal(outer_offsets_or_none[0], [0, 13, 17])
+    elif fill_type == FillType.ChunkCombinedOffsetOffset:
+        if TYPE_CHECKING:
+            filled = cast(cpy.FillReturn_ChunkCombinedOffsetOffset, filled)
+        assert_chunk_points(filled[0])
+        assert_chunk_offsets(filled[1])
+
+        outer_offsets_or_none = filled[2]
+        assert isinstance(outer_offsets_or_none, list) and len(outer_offsets_or_none) == 1
+        assert outer_offsets_or_none[0] is not None
+        assert_array_equal(outer_offsets_or_none[0], [0, 2, 3])
+    else:
+        raise RuntimeError(f"Unexpected fill_type {fill_type}")
 
 
 @pytest.mark.threads
 @pytest.mark.parametrize("fill_type", FillType.__members__.values())
 @pytest.mark.parametrize("name, thread_count",
                          [("serial", 1), ("threaded", 1), ("threaded", 2)])
-def test_return_by_fill_type_chunk(xyz_chunk_test, name, thread_count, fill_type):
+def test_return_by_fill_type_chunk(
+    xyz_chunk_test: tuple[cpy.CoordinateArray, ...],
+    name: str,
+    thread_count: int,
+    fill_type: FillType,
+) -> None:
     x, y, z = xyz_chunk_test
     cont_gen = contour_generator(
         x, y, z, name=name, fill_type=fill_type, chunk_count=2, thread_count=thread_count,
@@ -686,20 +749,20 @@ def test_return_by_fill_type_chunk(xyz_chunk_test, name, thread_count, fill_type
     util_test.assert_filled(filled, fill_type)
 
     # Expected points by chunk.
-    expected = (
-        [[0.0, 0.7], [0.0, 0.5], [1.0, 0.7], [2.0, 0.9], [2.0, 1.0], [2.0, 1.1], [1.5, 1.0],
-         [1.0, 0.9], [0.0, 0.7]],
-        [[2.0, 1.0], [2.0, 0.9], [2.5, 1.0], [3.0, 1.1], [4.0, 1.3], [4.0, 1.5], [3.0, 1.3],
-         [2.0, 1.1], [2.0, 1.0]],
-        [[1.5, 3.0], [2.0, 2.9], [2.0, 3.0], [2.0, 3.1], [1.0, 3.3], [0.0, 3.5], [0.0, 3.3],
-         [1.0, 3.1], [1.5, 3.0]],
-        [[2.0, 3.0], [2.0, 2.9], [3.0, 2.7], [4.0, 2.5], [4.0, 2.7], [3.0, 2.9], [2.5, 3.0],
-         [2.0, 3.1], [2.0, 3.0]],
-    )
+    expected: list[cpy.PointArray] = [
+        np.asarray([[0.0, 0.7], [0.0, 0.5], [1.0, 0.7], [2.0, 0.9], [2.0, 1.0], [2.0, 1.1],
+                    [1.5, 1.0], [1.0, 0.9], [0.0, 0.7]]),
+        np.asarray([[2.0, 1.0], [2.0, 0.9], [2.5, 1.0], [3.0, 1.1], [4.0, 1.3], [4.0, 1.5],
+                    [3.0, 1.3], [2.0, 1.1], [2.0, 1.0]]),
+        np.asarray([[1.5, 3.0], [2.0, 2.9], [2.0, 3.0], [2.0, 3.1], [1.0, 3.3], [0.0, 3.5],
+                    [0.0, 3.3], [1.0, 3.1], [1.5, 3.0]]),
+        np.asarray([[2.0, 3.0], [2.0, 2.9], [3.0, 2.7], [4.0, 2.5], [4.0, 2.7], [3.0, 2.9],
+                    [2.5, 3.0], [2.0, 3.1], [2.0, 3.0]]),
+    ]
 
-    if fill_type in (FillType.OuterCode, FillType.OuterOffset):
-        points = filled[0]
-        assert len(points) == 4
+    # Helper functions to avoid code duplication for the different FillTypes.
+    def assert_outer_points(points: list[cpy.PointArray], expected: list[cpy.PointArray]) -> None:
+        assert isinstance(points, list) and len(points) == 4
         if name == "threaded" and cont_gen.thread_count > 1:
             # Polygons may be in any order so sort lines and expected.
             points = util_test.sort_by_first_xy(points)
@@ -707,47 +770,94 @@ def test_return_by_fill_type_chunk(xyz_chunk_test, name, thread_count, fill_type
         for chunk in range(4):
             assert_allclose(points[chunk], expected[chunk])
 
-        if fill_type == FillType.OuterCode:
-            codes = filled[1]
-            for chunk in range(4):
-                assert_array_equal(codes[chunk], [1, 2, 2, 2, 2, 2, 2, 2, 79])
-        else:
-            offsets = filled[1]
-            for chunk in range(4):
-                assert_array_equal(offsets[chunk], [0, 9])
-    else:
-        points = filled[0]
-        assert len(points) == 4
+    def assert_chunk_points(
+        points_or_none: list[cpy.PointArray | None], expected: list[cpy.PointArray],
+    ) -> None:
+        assert isinstance(points_or_none, list) and len(points_or_none) == 4
         for chunk in range(4):
-            assert_allclose(points[chunk], expected[chunk])
+            chunk_points = points_or_none[chunk]
+            assert chunk_points is not None
+            assert_allclose(chunk_points, expected[chunk])
 
-        if fill_type in (FillType.ChunkCombinedCode, FillType.ChunkCombinedCodeOffset):
-            codes = filled[1]
-            for chunk in range(4):
-                assert_array_equal(codes[chunk], [1, 2, 2, 2, 2, 2, 2, 2, 79])
-        else:
-            offsets = filled[1]
-            for chunk in range(4):
-                assert_array_equal(offsets[chunk], [0, 9])
+    def assert_chunk_codes(codes_or_none: list[cpy.CodeArray | None]) -> None:
+        assert isinstance(codes_or_none, list) and len(codes_or_none) == 4
+        for chunk in range(4):
+            chunk_codes = codes_or_none[chunk]
+            assert chunk_codes is not None
+            assert_array_equal(chunk_codes, [1, 2, 2, 2, 2, 2, 2, 2, 79])
 
-        if fill_type == FillType.ChunkCombinedCodeOffset:
-            outer_offsets = filled[2]
-            for chunk in range(4):
-                assert_array_equal(outer_offsets[chunk], [0, 9])
-        elif fill_type == FillType.ChunkCombinedOffsetOffset:
-            outer_offsets = filled[2]
-            for chunk in range(4):
-                assert_array_equal(outer_offsets[chunk], [0, 1])
+    def assert_chunk_offsets(offsets_or_none: list[cpy.OffsetArray | None]) -> None:
+        assert isinstance(offsets_or_none, list) and len(offsets_or_none) == 4
+        for chunk in range(4):
+            chunk_offsets = offsets_or_none[chunk]
+            assert chunk_offsets is not None
+            assert_array_equal(chunk_offsets, [0, 9])
+
+    if fill_type == FillType.OuterCode:
+        if TYPE_CHECKING:
+            filled = cast(cpy.FillReturn_OuterCode, filled)
+        assert_outer_points(filled[0], expected)
+        codes = filled[1]
+        assert isinstance(codes, list) and len(codes) == 4
+        for chunk in range(4):
+            assert_array_equal(codes[chunk], [1, 2, 2, 2, 2, 2, 2, 2, 79])
+    elif fill_type == FillType.OuterOffset:
+        if TYPE_CHECKING:
+            filled = cast(cpy.FillReturn_OuterOffset, filled)
+        assert_outer_points(filled[0], expected)
+        offsets = filled[1]
+        assert isinstance(offsets, list) and len(offsets) == 4
+        for chunk in range(4):
+            assert_array_equal(offsets[chunk], [0, 9])
+    elif fill_type == FillType.ChunkCombinedCode:
+        if TYPE_CHECKING:
+            filled = cast(cpy.FillReturn_ChunkCombinedCode, filled)
+        assert_chunk_points(filled[0], expected)
+        assert_chunk_codes(filled[1])
+    elif fill_type == FillType.ChunkCombinedOffset:
+        if TYPE_CHECKING:
+            filled = cast(cpy.FillReturn_ChunkCombinedOffset, filled)
+        assert_chunk_points(filled[0], expected)
+        assert_chunk_offsets(filled[1])
+    elif fill_type == FillType.ChunkCombinedCodeOffset:
+        if TYPE_CHECKING:
+            filled = cast(cpy.FillReturn_ChunkCombinedCodeOffset, filled)
+        assert_chunk_points(filled[0], expected)
+        assert_chunk_codes(filled[1])
+
+        outer_offsets_or_none = filled[2]
+        assert isinstance(outer_offsets_or_none, list) and len(outer_offsets_or_none) == 4
+        for chunk in range(4):
+            chunk_outer_offsets = outer_offsets_or_none[chunk]
+            assert chunk_outer_offsets is not None
+            assert_array_equal(chunk_outer_offsets, [0, 9])
+    elif fill_type == FillType.ChunkCombinedOffsetOffset:
+        if TYPE_CHECKING:
+            filled = cast(cpy.FillReturn_ChunkCombinedOffsetOffset, filled)
+        assert_chunk_points(filled[0], expected)
+        assert_chunk_offsets(filled[1])
+
+        outer_offsets_or_none = filled[2]
+        assert isinstance(outer_offsets_or_none, list) and len(outer_offsets_or_none) == 4
+        for chunk in range(4):
+            chunk_outer_offsets = outer_offsets_or_none[chunk]
+            assert chunk_outer_offsets is not None
+            assert_array_equal(chunk_outer_offsets, [0, 1])
+    else:
+        raise RuntimeError(f"Unexpected fill_type {fill_type}")
 
 
 @pytest.mark.parametrize("name, fill_type", util_test.all_names_and_fill_types())
 @pytest.mark.parametrize("corner_mask", [None, False, True])
-def test_filled_random_big(name, fill_type, corner_mask):
+def test_filled_random_big(name: str, fill_type: FillType, corner_mask: bool | None) -> None:
     if corner_mask and name in ["mpl2005", "mpl2014"]:
         pytest.skip()
 
     x, y, z = random((1000, 1000), mask_fraction=0.0 if corner_mask is None else 0.05)
-    cont_gen = contour_generator(x, y, z, name=name, corner_mask=corner_mask, fill_type=fill_type)
+    cont_gen = contour_generator(
+        x, y, z, name=name, corner_mask=corner_mask if isinstance(corner_mask, bool) else False,
+        fill_type=fill_type,
+    )
     levels = np.arange(0.0, 1.01, 0.1)
 
     assert cont_gen.fill_type == fill_type
@@ -758,8 +868,8 @@ def test_filled_random_big(name, fill_type, corner_mask):
 
 
 @pytest.mark.slow
-@pytest.mark.parametrize('seed', np.arange(10))
-def test_filled_compare_slow(seed):
+@pytest.mark.parametrize("seed", np.arange(10))
+def test_filled_compare_slow(seed: int) -> None:
     x, y, z = random((1000, 1000), mask_fraction=0.05, seed=seed)
     levels = np.arange(0.0, 1.01, 0.1)
 
@@ -777,12 +887,30 @@ def test_filled_compare_slow(seed):
         filled_serial = cont_gen_serial.filled(levels[i], levels[i+1])
         util_test.assert_filled(filled_serial, cont_gen_serial.fill_type)
 
+        if TYPE_CHECKING:
+            filled_mpl2014 = cast(cpy.FillReturn_OuterCode, filled_mpl2014)
+            filled_serial = cast(cpy.FillReturn_ChunkCombinedOffsetOffset, filled_serial)
+
         # Check same results obtained for each in terms of number of points, etc.
         code_list = filled_mpl2014[1]
         n_points = reduce(add, map(len, code_list))
         n_outers = len(code_list)
         n_lines = reduce(add, map(lambda c: np.count_nonzero(c == 1), code_list))
 
+        assert filled_serial[0][0] is not None
         assert n_points == len(filled_serial[0][0])
+
+        assert filled_serial[1][0] is not None
         assert n_lines == len(filled_serial[1][0]) - 1
+
+        assert filled_serial[2][0] is not None
         assert n_outers == len(filled_serial[2][0]) - 1
+
+
+@pytest.mark.parametrize("name", util_test.all_names())
+@pytest.mark.parametrize("z", [np.nan, -np.nan, np.inf, -np.inf])
+@pytest.mark.parametrize("zlevel", [0.0, np.nan, -np.nan, np.inf, -np.inf])
+def test_filled_z_nonfinite(name: str, z: float, zlevel: float) -> None:
+    cont_gen = contour_generator(z=[[z, z], [z, z]], name=name, fill_type=FillType.OuterCode)
+    filled = cont_gen.filled(zlevel, zlevel)
+    assert filled == ([], [])
